/*
 * #%L
 * OME-XML Java library for working with OME-XML metadata structures.
 * %%
 * Copyright (C) 2006 - 2016 Open Microscopy Environment:
 *   - Massachusetts Institute of Technology
 *   - National Institutes of Health
 *   - University of Dundee
 *   - Board of Regents of the University of Wisconsin-Madison
 *   - Glencoe Software, Inc.
 * %%
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of any organization.
 * #L%
 */

/*-----------------------------------------------------------------------------
 *
 * THIS IS AUTOMATICALLY GENERATED CODE.  DO NOT MODIFY.
 *
 *-----------------------------------------------------------------------------
 */

package ome.xml.model;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import ome.xml.model.enums.*;
import ome.xml.model.enums.handlers.*;
import ome.xml.model.primitives.*;
import ome.units.quantity.*;
import ome.units.unit.Unit;

public class ROI extends AbstractOMEModelObject
{
  // Base:  -- Name: ROI -- Type: ROI -- modelBaseType: AbstractOMEModelObject -- langBaseType: Object

  // -- Constants --

  public static final String NAMESPACE = "http://www.openmicroscopy.org/Schemas/OME/2016-06";

  /** Logger for this class. */
  private static final Logger LOGGER =
    LoggerFactory.getLogger(ROI.class);

  // -- Instance variables --

  // ID property
  private String id;

  // Name property
  private String name;

  // Union property
  private Union union;

  // AnnotationRef reference (occurs more than once)
  private List<Annotation> annotationLinks = new ReferenceList<>();

  // Description property
  private String description;

  // Image_BackReference back reference (occurs more than once)
  private List<Image> imageLinks = new ReferenceList<>();

  // MicrobeamManipulation_BackReference back reference (occurs more than once)
  private List<MicrobeamManipulation> microbeamManipulationLinks = new ReferenceList<>();

  // Folder_BackReference back reference (occurs more than once)
  private List<Folder> folderLinks = new ReferenceList<>();

  // -- Constructors --

  /** Default constructor. */
  public ROI()
  {
  }



  /**
   * Constructs ROI recursively from an XML DOM tree.
   * @param element Root of the XML DOM tree to construct a model object
   * graph from.
   * @param model Handler for the OME model which keeps track of instances
   * and references seen during object population.
   * @throws EnumerationException If there is an error instantiating an
   * enumeration during model object creation.
   */
  public ROI(Element element, OMEModel model)
    throws EnumerationException
  {
    update(element, model);
  }

  /** Copy constructor. */
  public ROI(ROI orig)
  {
    id = orig.id;
    name = orig.name;
    union = orig.union;
    annotationLinks = orig.annotationLinks;
    description = orig.description;
    imageLinks = orig.imageLinks;
    microbeamManipulationLinks = orig.microbeamManipulationLinks;
    folderLinks = orig.folderLinks;
  }

  // -- Custom content from ROI specific template --


  // -- OMEModelObject API methods --

  /**
   * Updates ROI recursively from an XML DOM tree. <b>NOTE:</b> No
   * properties are removed, only added or updated.
   * @param element Root of the XML DOM tree to construct a model object
   * graph from.
   * @param model Handler for the OME model which keeps track of instances
   * and references seen during object population.
   * @throws EnumerationException If there is an error instantiating an
   * enumeration during model object creation.
   */
  public void update(Element element, OMEModel model)
    throws EnumerationException
  {
    super.update(element, model);
    if (!element.hasAttribute("ID") && getID() == null)
    {
      // TODO: Should be its own exception
      throw new RuntimeException(
        "ROI missing required ID property.");
    }
    if (element.hasAttribute("ID"))
    {
      // ID property
      setID(String.valueOf(
        element.getAttribute("ID")));
      // Adding this model object to the model handler
      model.addModelObject(getID(), this);
    }
    if (element.hasAttribute("Name"))
    {
      // Attribute property Name
      setName(String.valueOf(
        element.getAttribute("Name")));
    }
    List<Element> Union_nodeList =
      getChildrenByTagName(element, "Union");
    if (Union_nodeList.size() > 1)
    {
      // TODO: Should be its own Exception
      throw new RuntimeException(String.format(
        "Union node list size %d != 1",
        Union_nodeList.size()));
    }
    else if (Union_nodeList.size() != 0)
    {
      // Element property Union which is complex (has
      // sub-elements)
      setUnion(new Union(Union_nodeList.get(0), model));
    }
    // Element reference AnnotationRef
    List<Element> AnnotationRef_nodeList =
      getChildrenByTagName(element, "AnnotationRef");
    for (Element AnnotationRef_element : AnnotationRef_nodeList)
    {
      AnnotationRef annotationLinks_reference = new AnnotationRef();
      annotationLinks_reference.setID(AnnotationRef_element.getAttribute("ID"));
      model.addReference(this, annotationLinks_reference);
    }
    List<Element> Description_nodeList =
      getChildrenByTagName(element, "Description");
    if (Description_nodeList.size() > 1)
    {
      // TODO: Should be its own Exception
      throw new RuntimeException(String.format(
        "Description node list size %d != 1",
        Description_nodeList.size()));
    }
    else if (Description_nodeList.size() != 0)
    {
      // Element property Description which is not complex (has no
      // sub-elements)
      setDescription(
        String.valueOf(Description_nodeList.get(0).getTextContent()));
    }
  }

  // -- ROI API methods --

  public boolean link(Reference reference, OMEModelObject o)
  {
    if (reference instanceof AnnotationRef)
    {
      Annotation o_casted = (Annotation) o;
      o_casted.linkROI(this);
      annotationLinks.add(o_casted);
      return true;
    }
    return super.link(reference, o);
  }

  // Property ID
  public String getID()
  {
    return id;
  }

  public void setID(String id)
  {
    this.id = id;
  }

  // Property Name
  public String getName()
  {
    return name;
  }

  public void setName(String name)
  {
    this.name = name;
  }

  // Property Union
  public Union getUnion()
  {
    return union;
  }

  public void setUnion(Union union)
  {
    this.union = union;
  }

  // Reference which occurs more than once
  public int sizeOfLinkedAnnotationList()
  {
    return annotationLinks.size();
  }

  public List<Annotation> copyLinkedAnnotationList()
  {
    return new ArrayList<>(annotationLinks);
  }

  public Annotation getLinkedAnnotation(int index)
  {
    return annotationLinks.get(index);
  }

  public Annotation setLinkedAnnotation(int index, Annotation o)
  {
    return annotationLinks.set(index, o);
  }

  @SuppressWarnings("UnusedReturnValue")
  public boolean linkAnnotation(Annotation o)
  {

    o.linkROI(this);
    return annotationLinks.add(o);
  }

  @SuppressWarnings("UnusedReturnValue")
  public boolean unlinkAnnotation(Annotation o)
  {

    o.unlinkROI(this);
    return annotationLinks.remove(o);
  }

  // Property Description
  public String getDescription()
  {
    return description;
  }

  public void setDescription(String description)
  {
    this.description = description;
  }

  // Reference which occurs more than once
  public int sizeOfLinkedImageList()
  {
    return imageLinks.size();
  }

  public List<Image> copyLinkedImageList()
  {
    return new ArrayList<>(imageLinks);
  }

  public Image getLinkedImage(int index)
  {
    return imageLinks.get(index);
  }

  public Image setLinkedImage(int index, Image o)
  {
    return imageLinks.set(index, o);
  }

  @SuppressWarnings("UnusedReturnValue")
  public boolean linkImage(Image o)
  {
    return imageLinks.add(o);
  }

  @SuppressWarnings("UnusedReturnValue")
  public boolean unlinkImage(Image o)
  {
    return imageLinks.remove(o);
  }

  // Reference which occurs more than once
  public int sizeOfLinkedMicrobeamManipulationList()
  {
    return microbeamManipulationLinks.size();
  }

  public List<MicrobeamManipulation> copyLinkedMicrobeamManipulationList()
  {
    return new ArrayList<>(microbeamManipulationLinks);
  }

  public MicrobeamManipulation getLinkedMicrobeamManipulation(int index)
  {
    return microbeamManipulationLinks.get(index);
  }

  public MicrobeamManipulation setLinkedMicrobeamManipulation(int index, MicrobeamManipulation o)
  {
    return microbeamManipulationLinks.set(index, o);
  }

  @SuppressWarnings("UnusedReturnValue")
  public boolean linkMicrobeamManipulation(MicrobeamManipulation o)
  {
    return microbeamManipulationLinks.add(o);
  }

  @SuppressWarnings("UnusedReturnValue")
  public boolean unlinkMicrobeamManipulation(MicrobeamManipulation o)
  {
    return microbeamManipulationLinks.remove(o);
  }

  // Reference which occurs more than once
  public int sizeOfLinkedFolderList()
  {
    return folderLinks.size();
  }

  public List<Folder> copyLinkedFolderList()
  {
    return new ArrayList<>(folderLinks);
  }

  public Folder getLinkedFolder(int index)
  {
    return folderLinks.get(index);
  }

  public Folder setLinkedFolder(int index, Folder o)
  {
    return folderLinks.set(index, o);
  }

  @SuppressWarnings("UnusedReturnValue")
  public boolean linkFolder(Folder o)
  {
    return folderLinks.add(o);
  }

  @SuppressWarnings("UnusedReturnValue")
  public boolean unlinkFolder(Folder o)
  {
    return folderLinks.remove(o);
  }

  public Element asXMLElement(Document document)
  {
    return asXMLElement(document, null);
  }

  public Element asXMLElement(Document document, Element ROI_element)
  {
    // Creating XML block for ROI
    if (ROI_element == null)
    {
      ROI_element =
        document.createElementNS(NAMESPACE, "ROI");
    }

    // Ensure any base annotations add their Elements first
    super.asXMLElement(document, ROI_element);

    if (id != null)
    {
      // Attribute property ID
      ROI_element.setAttribute("ID", id);
    }
    if (name != null)
    {
      // Attribute property Name
      ROI_element.setAttribute("Name", name);
    }
    if (union != null)
    {
      // Element property Union which is complex (has
      // sub-elements)
      Element child =
        document.createElementNS(NAMESPACE, "Union");
      union.asXMLElement(document, child);
      ROI_element.appendChild(child);
    }
    if (annotationLinks != null)
    {
      // Reference property AnnotationRef which occurs more than once
      for (Annotation annotationLinks_value : annotationLinks)
      {
        AnnotationRef o = new AnnotationRef();
        o.setID(annotationLinks_value.getID());
        Element child =
          document.createElementNS(NAMESPACE, "AnnotationRef");
        o.asXMLElement(document, child);
        ROI_element.appendChild(child);
      }
    }
    if (description != null)
    {
    // Element property Description which is not complex (has no
    // sub-elements)
    Element description_element =
      document.createElementNS(NAMESPACE, "Description");
      description_element.setTextContent(description);
      ROI_element.appendChild(description_element);
    }
    // *** IGNORING *** Skipped back reference Image_BackReference
    // *** IGNORING *** Skipped back reference MicrobeamManipulation_BackReference
    // *** IGNORING *** Skipped back reference Folder_BackReference

    return ROI_element;
  }
}
